<?php
/**
 * Implements hook_help().
 */
function chessboard_help($path, $arg) {

  switch ($path) {
    case 'admin/help#chessboard':
      $output = t('Renders chessboard diagrams specified with the FEN syntax or a simple piece placement format.');
      break;

    default:
      $output = '';
      break;
  }

  return $output;
}

/**
 * Implements hook_init().
 */
function chessboard_init() {
  $formats = array();
  $result = db_query('SELECT * FROM {filter_formats}');
  while ($format = db_fetch_object($result)) {
    $formats[$format->format] = $format;
  }

  // Build list cache IDs.
  $cids = _chessboard_filter_page_structures_list_cids($formats);

  // Retrieve lists.
  $cache = _chessboard_cache_get_multiple($cids, 'cache_filter');

  // Build structure cache IDs.
  $cids = array();
  foreach ($cache as $item) {
    $cids += array_keys($item->data);
  }

  // Retrieve structures.
  $cache = _chessboard_cache_get_multiple($cids, 'cache_filter');

  // Process structures.
  foreach ($cache as $item) {
    if (isset($item->data['#attached'])) {
      _chessboard_process_attached($item->data);
    }
  }
}

/**
 * Implements hook_filter_tips().
 */
function chessboard_filter_tips($delta, $format, $long=FALSE) {
  return _chessboard_filter_0_tips(NULL, NULL, $long);
}

/**
 * Implements hook_filter().
 */
function chessboard_filter($op, $delta = 0, $format = -1, $text = '', $cache_id = 0) {
  static $filter;
  if (!isset($filter)) {
    $filter = new stdClass();
  }

  switch ($op) {
    case 'list':
      return array(0 => t('Chessboard'));

    case 'description':
      return t('Renders chessboard diagrams.');

    case 'process':
      $filter->format = $format;
      $filter->settings = array(
        // Borders.
        'border' => array(
          'T' => FALSE,
          'B' => FALSE,
          'L' => FALSE,
          'R' => FALSE,
        ),
        // Number of files: any integer (default: 8).
        'file_max' => 8,
        // Color of the upper left square: [l]ight (default), or [d]ark.
        'square_color_first' => 0,
      );

      return _chessboard_filter_0($text, $filter);

    default:
      return $text;
  }
}

/**
 * Chessboard filter. Provides filtering of input into chessboard diagrams.
 *
 * @param $filter
 *   The filter object containing settings for the given format.
 *
 * @param $text
 *   The content to filter.
 */
function _chessboard_filter_0($text, $filter) {
  // Prepare the callback function.
  _chessboard_filter_callback(NULL, $filter);

  return preg_replace_callback('@\[chessboard\](.*?)\[/chessboard\]@si', '_chessboard_filter_callback', $text);
}

/**
 * Filter tips callback for chessboard filter.
 */
function _chessboard_filter_0_tips($filter, $format, $long = FALSE) {
  if ($long) {
    return t('<h4>Chessboard Diagrams</h4>
<h5>Basic Usage</h5>
<p>Use the pair of <code>[chessboard]</code> and <code>[/chessboard]</code> tags to define a chessboard. The following formats are recognized:</p>
<ul>
  <li>The piece placement field (i.e., the first field) of the FEN (Forsyth-Edwards Notation) syntax. E.g., for the starting position:
    <code>[chessboard]rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR[/chessboard]</code>
  </li>
  <li>A simple and direct piece placement format. E.g., for the starting position:
<pre>[chessboard]
rnbqkbnr
pppppppp
--------
--------
--------
--------
PPPPPPPP
RNBQKBNR
[/chessboard]</pre>
Note that a dash (-) character represents an empty square.
  </li>
  <li>A mixture of the previous two formats. This is possible because the previous formats are in fact compatible to each other. E.g., for the starting position:
<pre>[chessboard]
rnbqkbnr
pppppppp
8/8/8/8
PPPPPPPP
RNBQKBNR
[/chessboard]</pre>
  However, the use of this format is NOT encouraged as it may lead to confusion.
  </li>
</ul>
<p>The renderer also supports a special feature that will converts \'x\' character to a marked square. For example, to display the diagram below which shows the squares a knight can control, you can write</p>
<pre>[chessboard]
--------
---x-x--
--x---x-
----N---
--x---x-
---x-x--
--------
--------
[/chessboard]</pre>
<p>or</p>
<pre>[chessboard]8/3x1x2/2x3x1/4N3/2x3x1/3x1x2/8/8[/chessboard]</pre>

<h5>Advanced Usage</h5>
<p>You can configure the appearance of the chessboard with a few flags, with the following syntax:</p>
<pre>[chessboard](<i>flags</i>)board[/chessboard]</pre>
<p>The following flags are supported:</p>
<ul>
  <li>Borders. Valid flags are T, B, L, and R, which correspond to the top border, the bottom border, the left border, and the right border respectively. The corresponding borders of the chessboard will be rendered for the flags specified. (Default: none is specified.)</li>
  <li>Number of Files. Valid flags are positive integers such as 4, 6, and 12. (Default: 8.)</li>
  <li>Color of the Upper Left Square. Valid flags are d and l, which correspond to the dark color and the light color. (Default: l.)</li>
</ul>
<p>These flags are especially useful in displaying a portion of a chessboard. For example, the following sentence illustrates a smothered mate at the upper-right corner:</p>
<pre>[chessboard](d3TR)
-rk
Npp
---
[/chessboard]
</pre>
<p>Any portion of a chessboard can be displayed by using these flags.</p>
');
  }
  else {
    return t('Renders chessboard diagrams.');
  }
}

/**
 * preg_replace callback to make chessboard diagrams out of chessboard syntax.
 *
 * @param $matches
 *   An array of matches found by preg_replace_callback(). Elements 0 and 1 of
 *   $matches must be the complete match and the piece placement field.
 *
 * @param $set_filter
 *   The filter object containing settings for the given format. This should
 *   only be set during a preparatory call before preg_replace_callback().
 *
 * @return
 *   A <code>string</code> containing the replacement text.
 */
function _chessboard_filter_callback($matches, $set_filter = NULL) {
  static $filter;
  if (isset($set_filter)) {
    // This is a preparatory call.
    $filter = $set_filter;
    return;
  }
  $item = array(
    'value' => $matches[1],
  );
  $item = chessboard_process_chessboard_notation($item, $filter->settings);
  $items = array($item);
  $element = _chessboard_filter_formatter_view($items);
  $output = _chessboard_render_elements($element);
  foreach (element_children($element) as $delta) {
    if (isset($element[$delta]['#attached']) && filter_format_allowcache($filter->format)) {
      // Store attached structures so we can process these when cached filtered text is being output.
      $data['#attached'] = $element[$delta]['#attached'];
      $cid = $filter->format . ':chessboard-filter-attached' . md5(serialize($element[$delta]['#attached']));
      $expire = CACHE_PERMANENT;
      cache_set($cid, $data, 'cache_filter', $expire);
      $list_cid = $filter->format . ':chessboard-filter-attached-cids';
      $cache = cache_get($list_cid, 'cache_filter');
      $cids = isset($cache->data) ? $cache->data : array();
      $cids[$cid] = TRUE;
      if (!$cache || $cids != $cache->data || $expire != $cache->expire) {
        cache_set($list_cid, $cids, 'cache_filter', $expire);
      }
    }
  }
  return $output;
}

function _chessboard_filter_page_structures_list_cids($formats) {
  $cids = array();
  foreach ($formats as $format) {
    if ($format->cache) {
      $cids[] = $format->format . ':chessboard-filter-attached-cids';
    }
  }
  return $cids;
}

/**
 * Build a renderable array for a chessboard value.
 *
 * @param $items
 *   Array of values.
 *
 * @return
 *   A renderable array for the $items, as an array of child elements keyed
 *   by numeric indexes starting from 0.
 */
function _chessboard_filter_formatter_view($items) {
  $element = array();
  foreach ($items as $delta => $item) {
    $element[$delta] = array(
      '#item' => $item,
      '#type' => 'chessboard_diagram',
      '#weight' => $delta,
      '#board' => $item['board'],
      '#border' => $item['border'],
      '#file_max' => $item['file_max'],
      '#square_color_first' => $item['square_color_first'],
    );
  }
  return $element;
}
